/************************************************************************/
/*																		*/
/*      File:           Bugaboo.C                                       */
/*      Author:         Paul Robson                                     */
/*		Purpose:		Game Shell          							*/
/*      Created:        20th December 2001								*/
/*      Modified:       20th December 2001								*/
/*																		*/
/************************************************************************/

#include "bugaboo.h"					/* Project include file */

static GFXDESC GfxDesc;					/* Graphics descriptor */
static GAME	   Game;					/* Game descriptor */
static int	   BestTime;				/* Best time to complete */

static int _PlayGame(int DragonTime);
static void _Title(void);

/************************************************************************/
/*																		*/
/*								Main Program							*/
/*																		*/
/************************************************************************/

int main()
{
	int Level,n,Keybd;
	GFXInitialise(&GfxDesc);			/* Initialise system */
    BestTime = 99*60+59;				/* Set best time */
    do
    {
    	while (GFXKeyboard() != 0) {}  	/* Wait for release */
        _Title();						/* Display the title */
    	while (Keybd = (GFXKeyboard() &	/* Wait for abandon/game */
    				(BTN_GAME|BTN_ABANDON)),Keybd == 0) rand();

	    if (Keybd == BTN_GAME)			/* Play game */
	    {
        	Level = 0;					/* Easiest (2 mins) */
			do
            {
            	Level++;				/* BUmp level */
            	n = 135-Level*15;		/* Calculate time */
                if (n < 30) n = 30;		/* Min of 30secs to dragon */
	    		n = _PlayGame(n);		/* Play a round */
                if (n>0 && n<BestTime)	/* Exceeded best time */
                				BestTime = n;
			} while (n != 0);			/* Keep going while successful */
		}
	}
    while (Keybd == BTN_GAME);
    return 0;
}

/************************************************************************/
/*																		*/
/*						  Display the title page						*/
/*																		*/
/************************************************************************/

#define CENTRE(y,c,s) GFXText(128-strlen(s)*4,y*8,c,0,s)

static void _Title(void)
{
	char _Temp[32];
	GFXText(0,0,0,0,"\x2");
    GFXText(128-202/2,10,0,0,NULL);
    CENTRE(9.5,10,"Written by Paul Robson \x7F 2001");
    CENTRE(11,10,"EMail: autismuk@aol.com");
    sprintf(_Temp,"Best time: %d:%2d",BestTime/60,BestTime%60);
    CENTRE(13,9,_Temp);
    CENTRE(15,14,"From the original Spectrum version");
    CENTRE(16.5,14,"by Paco Suarez and Paco Poralo");
    CENTRE(18,11,"Published by Quicksilva/Indescomp");
    CENTRE(20,12,  "1  Jump Left        0 Jump Right  ");
    CENTRE(21.5,12,"G  Start Game       A Abandon Game");
}

/************************************************************************/
/*																		*/
/*			Play the Game, return 0 or seconds to completion			*/
/*																		*/
/************************************************************************/

static int _PlayGame(int DragonTime)
{
    Game.State = ST_INITIALISE;			/* Setup state */
    Game.ClockDragonTime = GFXTimer()	/* Time the dragon comes out */
    					+ (long)DragonTime * 1000L;
    while (Game.State != ST_WIN &&		/* Keep running until win or lose */
    			Game.State != ST_LOST)
							GAMState(&Game,&GfxDesc);
	return (Game.State == ST_WIN) ?		/* Return the score */
							Game.CompletionTime : 0;
}


