/************************************************************************/
/*																		*/
/*      File:           Bugaboo.H                                       */
/*      Author:         Paul Robson                                     */
/*		Purpose:		Generic Include File							*/
/*      Created:        19th December 2001								*/
/*      Modified:       19th December 2001								*/
/*																		*/
/************************************************************************/

#include <stdio.h>						/* Standard includes */
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define  FPS			(200)			/* Moves per second */

#define  XSTART			(416)			/* Initial horizontal position */
#define  YSTART			(320)			/* Initial vertical position */

#define  GRAVITY		(0.03)

#define  ISSOLID(d,x,y) (GFXReadMap(d,x,y) >= 8)

#define  ST_INITIALISE	(0)				/* Initialising state */
#define  ST_FALLING		(1)				/* Falling vertically state */
#define  ST_WAITING		(2)				/* Waiting state */
#define  ST_CROUCH		(3)				/* Crouching state */
#define  ST_JUMPING		(4)				/* Jumping State (up) */
#define  ST_WIN			(6)				/* Win State */
#define  ST_EATEN		(7)				/* Eaten by Dragon State */
#define  ST_LOST   		(8)				/* Lose State */
#define  ST_CRASHED		(9)				/* Flea dazed */
#define  ST_TOPPLING	(10)			/* About to fall off edge */

#define  FLEA_ATEASE	(0)				/* Flea graphic IDs */
#define  FLEA_CROUCHED	(1)
#define  FLEA_JUMPING	(2)
#define  FLEA_CRASHED	(3)
#define  FLEA_FALLING	(4)

#define  COL_LIGHTOFF	(11)			/* Light off colour */
#define  COL_LIGHTON	(12)			/* Light on colour */

#define  BTN_ABANDON	(0x01)			/* Button bits */
#define  BTN_GAME		(0x02)
#define  BTN_JUMPLEFT	(0x04)
#define  BTN_JUMPRIGHT	(0x08)

typedef struct _GfxDesc					/* Describes the display */
{
	int xFlea,yFlea;					/* Flea position */
    int FleaChar; 						/* Flea graphic */
    int FleaRight;						/* Flea Right-facing flag */
    int xDragon,yDragon;				/* Dragon position */
    int DragonRight;					/* Dragon right-facing flag */
    int Caught;							/* Dragon caught flea ? */
    int xTop,yTop;						/* Position of background, top left */
	int	xSize,ySize;					/* Map Size */
    unsigned char *Map;					/* Pointer to Map Data */
} GFXDESC;

typedef struct _Game					/* Game state */
{
	int  State;							/* State of game */
    double FleaX,FleaY;					/* Flea current position */
    double FleaXV,FleaYV;				/* Velocity */
    double DragonX,DragonY;				/* Dragon position */
    int  DragonRight;					/* Dragon is right facing ? */
    int  DragonInitialised;				/* Dragon initialised flag */
    int  FleaRight;						/* Flea is right facing ? */
    long ClockStart;					/* Clock value at the start */
    long ClockEvent;					/* Clock value for next event */
    long ClockSynchronise;				/* Clock value for speed sync */
    long ClockDragonTime;				/* Time the dragon is unleashed */
    long LastClockDrawn;				/* Last seconds clock value drawn */
    int  LastLevelDrawn;				/* Last level in game value drawn */
    int  FirstDrawn;					/* Used to force a redraw */
    int	 JumpCount;						/* Jumping counter */
    int  CompletionTime;				/* Completion time */
} GAME;

void GFXInitialise(GFXDESC *Desc);		/* Prototyping */
int GFXReadMap(GFXDESC *Desc,int x,int y);
void GFXDraw(GFXDESC *Desc);
void GFXText(int x,int y,int Ink,int Paper,char *Text);
long GFXTimer(void);
int  GFXKeyboard(void);
int  GFXCheckBitmap(GFXDESC *d,int x,int y);
void GAMState(GAME *g,GFXDESC *d);
