/************************************************************************/
/*																		*/
/*      File:           Graphics.C                                      */
/*      Author:         Paul Robson                                     */
/*		Purpose:		All I/O Functions								*/
/*      Created:        19th December 2001								*/
/*      Modified:       20th December 2001								*/
/*																		*/
/************************************************************************/

#include "bugaboo.h"					/* Project include file */
#include "allegro.h"

static BITMAP *GfxBitmap;				/* The graphics bitmap */
static BITMAP *Background;				/* Background Map */
static BITMAP *Buffer;					/* Double Buffer */
static long   TickCount = 0;			/* Timer counter */

static unsigned char GameMap[65536];	/* Memory space for map */

static void _DrawBitmap(BITMAP *Bitmap,int x,int y,int Char);

/************************************************************************/
/*																		*/
/*							Timer Functions								*/
/*																		*/
/************************************************************************/

void GFXTimerFunc(void)	{ TickCount++; }
END_OF_FUNCTION(GFXTimerFunc);
long GFXTimer(void) { return TickCount; }

/************************************************************************/
/*																		*/
/*					Initialise the graphics and I/O						*/
/*																		*/
/************************************************************************/

void GFXInitialise(GFXDESC *Desc)
{
	RGB Palette[256];					/* Palette temp */
    int x,y;
    FILE *f;
	allegro_init();						/* Initialise Allegro */
    install_keyboard();					/* Install keyboard handler */
    install_timer();					/* Install the timer routine */
    LOCK_VARIABLE(TickCount);			/* Stop swapping out */
    LOCK_FUNCTION(GFXTimerFunc);
    install_int(GFXTimerFunc,1);		/* Call the timer at 1khz */
    set_color_depth(8);					/* To 320x200x8 Mode */
    set_gfx_mode(GFX_AUTODETECT,320,200,320,200);
    GfxBitmap=load_bitmap("bugaboo.pcx",/* Read in the graphics file */
    								Palette);
	f = fopen("Bugaboo.dat","rb");		/* Open data file */
	Desc->xSize = fgetc(f);				/* Read sizes */
    Desc->ySize = fgetc(f);
    fread(GameMap,1,sizeof(GameMap),f);	/* Read the game map */
    Desc->Map = GameMap;				/* Set up pointer */
	fclose(f);

	Background =						/* Create the background bitmap */
			create_system_bitmap(Desc->xSize*8,Desc->ySize*8);
    if (Background == NULL)
		Background = create_bitmap(Desc->xSize*8,Desc->ySize*8);

	Buffer =							/* Create the back buffer */
		create_system_bitmap(320,200);
    if (Background == NULL)
		create_bitmap(320,200);

	for (x = 0;x < Desc->xSize;x++)		/* Draw the background bitmap */
    	for (y = 0;y < Desc->ySize;y++)
        	_DrawBitmap(Background,x,y,GFXReadMap(Desc,x*8,y*8));

}

/************************************************************************/
/*																		*/
/*					   Read a character off the map						*/
/*																		*/
/************************************************************************/

int GFXReadMap(GFXDESC *Desc,int x,int y)
{
	return (int)(Desc->Map[(x>>3)+(y>>3)*Desc->xSize]);
}

/************************************************************************/
/*																		*/
/*				Draw one of the tile bitmaps on a bitmap				*/
/*																		*/
/************************************************************************/

static void _DrawBitmap(BITMAP *Bitmap,int x,int y,int Char)
{
	blit(GfxBitmap,Bitmap,
		 (Char % 32)*8+8,(Char / 32)*8+8,
		 x*8,y*8,8,8);
}

/************************************************************************/
/*																		*/
/*						Redraw the whole display						*/
/*																		*/
/************************************************************************/

void GFXDraw(GFXDESC *Desc)
{
	int n,x,y;
	Desc->xTop = Desc->xFlea+8-256/2;	/* Calculate bitmap position */
    Desc->yTop = Desc->yFlea+8-176/2;
	if (Desc->xTop < 0) Desc->xTop = 0;	/* Check in legitimate range */
    if (Desc->yTop < 0) Desc->yTop = 0;
    n = Desc->xSize * 8 - 256;			/* Max value of x */
    if (Desc->xTop > n) Desc->xTop = n;
    n = Desc->ySize * 8 - 176;			/* Max value of y */
    if (Desc->yTop > n) Desc->yTop = n;

	blit(Background,Buffer,Desc->xTop,Desc->yTop,32,4,256,176);

    x = -Desc->xTop+32;y = -Desc->yTop+4;

    if (Desc->Caught == 0)
    {
		masked_blit(GfxBitmap,
					Buffer,
				    (Desc->FleaRight) ? 296:280,
				    8 + Desc->FleaChar * 16,
			    	Desc->xFlea+x,Desc->yFlea+y,16,16);
	}

	masked_blit(GfxBitmap,
    			Buffer,
                Desc->DragonRight ? 264:216,
                Desc->Caught ? 102:134,
                Desc->xDragon+x,Desc->yDragon+y,
                48,32);

	blit(Buffer,screen,32,4,32,4,256,176);
}

/************************************************************************/
/*																		*/
/*							Draw a text string							*/
/*																		*/
/************************************************************************/

void GFXText(int x,int y,int Ink,int Paper,char *Text)
{
	int c,x1,y1,xc,yc;
    if (Text == NULL) 					/* NULL prints BUGABOO Logo */
    {
    	masked_blit(GfxBitmap,screen,8,104,x+32,y+4,202,61);
    	return;
    }
	while (*Text != '\0')				/* Until finished */
    {
    	c = *Text++;					/* Get character */
        if (c == 1)						/* 1 is circle */
        {
        	rectfill(screen,x+32,y+4,x+39,y+11,Paper);
            circlefill(screen,x+36,y+8,3,Ink);
        }
        if (c == 2)						/* 2 clear */
            clear(screen);
        if (c >= 32 && c < 128)			/* If legitimate... */
        {
        	xc = (c-32)%32*8+8;
            yc = (c-32)/32*8+72;
            for (x1 = 0;x1 < 8;x1++)
            	for (y1 = 0;y1 < 8;y1++)
                	putpixel(screen,
                			 x+x1+32,y+y1+4,
							 getpixel(GfxBitmap,xc+x1,yc+y1) ? Ink:Paper);
        }
        x = x+8;						/* Next position on */
    }
}

/************************************************************************/
/*																		*/
/*								Scan keyboard							*/
/*																		*/
/************************************************************************/

int GFXKeyboard(void)
{
	int n = 0;
    if (key[KEY_A]) n |= BTN_ABANDON;
    if (key[KEY_G]) n |= BTN_GAME;
    if (key[KEY_0]) n |= BTN_JUMPRIGHT;
    if (key[KEY_1]) n |= BTN_JUMPLEFT;
    return n;
}

/************************************************************************/
/*																		*/
/*			   Check to see if a draw will cause a collision			*/
/*																		*/
/************************************************************************/

int  GFXCheckBitmap(GFXDESC *d,int x,int y)
{
	int n;
    int x1 = 5,x2 = 14,y1 = 2,y2 = 15;
    for (n = x1;n < x2;n++)
    {
    	if (getpixel(Background,n,y1) != 0) return 1;
    	if (getpixel(Background,n,y2) != 0) return 1;
	}
    for (n = y1;n < y2;n++)
    {
    	if (getpixel(Background,x1,n) != 0) return 1;
    	if (getpixel(Background,x2,n) != 0) return 1;
	}
	return 0;
}

BEGIN_DIGI_DRIVER_LIST
END_DIGI_DRIVER_LIST
BEGIN_MIDI_DRIVER_LIST
END_MIDI_DRIVER_LIST
BEGIN_JOYSTICK_DRIVER_LIST
END_JOYSTICK_DRIVER_LIST

